% Pricing 3-year American and European put options on a discount bond 
% using Hull White trinomial tree.
% Reproduces Figure 9.17 in Clewlow and Strickland's book
% "Implementing Derivatives Models".

clc; clear;

% Yearly zero coupon rates and tenors
Rates = [.05 .0575 .0625 .0675 .07 .0725];
Tenors = [1:6];

% Maturity (T) number of time steps (N) and time increment (dt)
T = 5;
N = 5;
dt = T/N;
Time = 1:T+1;
R = Rates(1:T+1);

% Discount factor.
P = exp(-R.*Time);

% Strike price for puts on the discount bond
K = 0.85;

% Hull White tree parameters
alpha = 0.1;
sigma = 0.01;
dr = sigma*sqrt(3*dt);
dx = dr;

% Threshold (M), jMax, and state vector (x)
M = -alpha*dt;
jMax = ceil(-0.1835/M);
x = dr.*[jMax:-1:-jMax];

% Build the HW probability trees for pu, pm, and pd.
[pu pm pd] = BuildHWprobs(jMax,N,M);

% Construct the Hull-White interest rate tree (r), 
% discount factors (d) and Arrow Debreu prices (Q).
[r d Q] = BuildHWtree(R,P,jMax,N,x,dx,dt,pu,pm,pd)

% Initialize the discount bond B.
B = zeros(2*jMax+1,N+1);

% Last column of discount bond is 1.
B(:,end) = 1;

% Work backwards through tree to get remaining discount bond prices
for j=N:-1:1
	if j>jMax
		for i=1:2*jMax+1
			if i==1             % first row.
				B(i,j) = d(i,j)*(B(i,j+1)*pu(i,j) + B(i+1,j+1)*pm(i,j) + B(i+2,j+1)*pd(i,j));
			elseif i==2*jMax+1  % last row.
				B(i,j) = d(i,j)*(B(i,j+1)*pd(i,j) + B(i-1,j+1)*pm(i,j) + B(i-2,j+1)*pu(i,j));
			else
				B(i,j) = d(i,j)*(B(i-1,j+1)*pu(i,j) + B(i,j+1)*pm(i,j) + B(i+1,j+1)*pd(i,j));
			end
		end
	else
		for i=jMax-(j-2):jMax+j
			B(i,j) = d(i,j)*(B(i-1,j+1)*pu(i,j) + B(i,j+1)*pm(i,j) + B(i+1,j+1)*pd(i,j));
		end
	end
end

% Print out the Discount bond (middle of Figure 9.17)
B

% Price the American and European puts.  
% Maturity is three years.
Mat = 3;

% European put is the payoff times the Arrow Debreu prices
Payoff = max(K - B(:,Mat+1), 0);
EuroPut = Q(:,Mat+1)' * Payoff

% Initialize the American puts
AP = zeros(2*jMax+1,Mat+1);

% Intrinsic value at maturity.
AP(:,Mat+1) = max(K-B(:,Mat+1),0);
EP = AP;

% Work backwards through the tree
for j=Mat:-1:1
	if j>jMax   % Box of the tree
		for i=1:2*jMax+1
			if i==1             % first row.
				EP(i,j) = d(i,j)*(EP(i,j+1)*pu(i,j) + EP(i+1,j+1)*pm(i,j) + EP(i+2,j+1)*pd(i,j));
			elseif i==2*jMax+1  % last row.
				EP(i,j) = d(i,j)*(EP(i,j+1)*pd(i,j) + EP(i-1,j+1)*pm(i,j) + EP(i-2,j+1)*pu(i,j));
			else                % middle rows
				EP(i,j) = d(i,j)*(EP(i-1,j+1)*pu(i,j) + EP(i,j+1)*pm(i,j) + EP(i+1,j+1)*pd(i,j));
			end
			AP(i,j) = max(EP(i,j), K - B(i,j));
		end
	else       % Tip of the tree
		for i=jMax-(j-2):jMax+j
			EP(i,j) = d(i,j)*(EP(i-1,j+1)*pu(i,j) + EP(i,j+1)*pm(i,j) + EP(i+1,j+1)*pd(i,j));
			AP(i,j) = max(EP(i,j), K - B(i,j));
		end
	end
end

% Print out the American put tree.
AP

% Print out the American put price
AmerPut = AP(jMax+1,1)

